@@ -35,6 +35,7 @@ struct connman_device {
 	enum connman_device_type type;
 	enum connman_device_mode mode;
 	enum connman_device_policy policy;
+	connman_bool_t secondary;
 	connman_bool_t powered;
 	connman_bool_t carrier;
 	connman_bool_t scanning;
@@ -45,6 +46,7 @@ struct connman_device {
 	char *node;
 	char *address;
 	char *interface;
+	char *ident;
 	unsigned int connections;
 	guint scan_timeout;
 
@@ -159,6 +161,61 @@ static enum connman_device_policy string2policy(const char *policy)
 		return CONNMAN_DEVICE_POLICY_UNKNOWN;
 }
 
+static int set_carrier(struct connman_device *device, connman_bool_t carrier)
+{
+	struct connman_service *service;
+
+	service = __connman_service_lookup_from_device(device);
+	__connman_service_set_carrier(service, carrier);
+
+	if (carrier == TRUE) {
+		enum connman_element_type type = CONNMAN_ELEMENT_TYPE_UNKNOWN;
+		struct connman_element *element;
+
+		device->disconnected = TRUE;
+
+		switch (device->policy) {
+		case CONNMAN_DEVICE_POLICY_UNKNOWN:
+		case CONNMAN_DEVICE_POLICY_IGNORE:
+		case CONNMAN_DEVICE_POLICY_OFF:
+		case CONNMAN_DEVICE_POLICY_MANUAL:
+			return 0;
+		case CONNMAN_DEVICE_POLICY_AUTO:
+			break;
+		}
+
+		switch (device->element.ipv4.method) {
+		case CONNMAN_IPV4_METHOD_UNKNOWN:
+		case CONNMAN_IPV4_METHOD_OFF:
+			return 0;
+		case CONNMAN_IPV4_METHOD_STATIC:
+			type = CONNMAN_ELEMENT_TYPE_IPV4;
+			break;
+		case CONNMAN_IPV4_METHOD_DHCP:
+			type = CONNMAN_ELEMENT_TYPE_DHCP;
+			break;
+		}
+
+		element = connman_element_create(NULL);
+		if (element != NULL) {
+			element->type  = type;
+			element->index = device->element.index;
+
+			if (connman_element_register(element,
+							&device->element) < 0)
+				connman_element_unref(element);
+
+			device->disconnected = FALSE;
+
+			__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_CONFIGURATION);
+		}
+	} else
+		connman_element_unregister_children(&device->element);
+
+	return 0;
+}
+
 static int set_powered(struct connman_device *device, connman_bool_t powered)
 {
 	struct connman_device_driver *driver = device->driver;
@@ -178,6 +235,8 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 	} else {
 		g_hash_table_remove_all(device->networks);
 
+		set_carrier(device, FALSE);
+
 		if (driver->disable) {
 			err = driver->disable(device);
 			__connman_notifier_device_type_decrease(device->type);
@@ -215,6 +274,8 @@ static int set_policy(DBusConnection *connection,
 	case CONNMAN_DEVICE_POLICY_MANUAL:
 		if (device->powered == FALSE)
 			err = set_powered(device, TRUE);
+		else
+			err = set_carrier(device, device->carrier);
 		break;
 	}
 
@@ -298,14 +359,9 @@ static DBusMessage *get_properties(DBusConnection *conn,
 			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
 			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
 
-	str = type2description(device->type);
-	if (str != NULL && device->interface != NULL) {
-		char *name = g_strdup_printf("%s (%s)", str, device->interface);
-		if (name != NULL)
-			connman_dbus_dict_append_variant(&dict, "Name",
-						DBUS_TYPE_STRING, &name);
-		g_free(name);
-	}
+	if (device->name != NULL)
+		connman_dbus_dict_append_variant(&dict, "Name",
+					DBUS_TYPE_STRING, &device->name);
 
 	str = type2string(device->type);
 	if (str != NULL)
@@ -397,7 +453,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 		err = set_powered(device, powered);
 		if (err < 0 && err != -EINPROGRESS)
-			return __connman_error_failed(msg);
+			return __connman_error_failed(msg, -err);
 	} else if (g_str_equal(name, "Policy") == TRUE) {
 		enum connman_device_policy policy;
 		const char *str;
@@ -413,7 +469,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 		err = set_policy(conn, device, policy);
 		if (err < 0)
-			return __connman_error_failed(msg);
+			return __connman_error_failed(msg, -err);
 	} else if (g_str_equal(name, "Priority") == TRUE) {
 		connman_uint8_t priority;
 
@@ -501,7 +557,7 @@ static DBusMessage *join_network(DBusConnection *conn,
 
 	network = connman_network_create("00_00_00_00_00_00", type);
 	if (network == NULL)
-		return __connman_error_failed(msg);
+		return __connman_error_failed(msg, ENOMEM);
 
 	while (dbus_message_iter_get_arg_type(&array) == DBUS_TYPE_DICT_ENTRY) {
 		DBusMessageIter entry, value;
@@ -537,7 +593,7 @@ static DBusMessage *join_network(DBusConnection *conn,
 	connman_network_unref(network);
 
 	if (err < 0)
-		return __connman_error_failed(msg);
+		return __connman_error_failed(msg, -err);
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
@@ -587,11 +643,11 @@ static DBusMessage *propose_scan(DBusConnection *conn,
 		return __connman_error_not_supported(msg);
 
 	if (device->powered == FALSE)
-		return __connman_error_failed(msg);
+		return __connman_error_failed(msg, EINVAL);
 
 	err = device->driver->scan(device);
 	if (err < 0)
-		return __connman_error_failed(msg);
+		return __connman_error_failed(msg, -err);
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
@@ -739,7 +795,8 @@ static int setup_device(struct connman_device *device)
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
 		break;
 	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
-		__connman_profile_add_device(device);
+		if (device->secondary == FALSE)
+			__connman_profile_add_device(device);
 		break;
 	}
 
@@ -783,7 +840,8 @@ static void remove_device(struct connman_device *device)
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
 		break;
 	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
-		__connman_profile_remove_device(device);
+		if (device->secondary == FALSE)
+			__connman_profile_remove_device(device);
 		break;
 	}
 
@@ -880,6 +938,7 @@ static void device_destruct(struct connman_element *element)
 
 	DBG("element %p name %s", element, element->name);
 
+	g_free(device->ident);
 	g_free(device->node);
 	g_free(device->name);
 	g_free(device->address);
@@ -929,9 +988,11 @@ struct connman_device *connman_device_create(const char *node,
 
 	device->element.ipv4.method = CONNMAN_IPV4_METHOD_DHCP;
 
-	device->type   = type;
-	device->mode   = CONNMAN_DEVICE_MODE_UNKNOWN;
-	device->policy = CONNMAN_DEVICE_POLICY_AUTO;
+	device->type      = type;
+	device->name      = g_strdup(type2description(device->type));
+	device->mode      = CONNMAN_DEVICE_MODE_UNKNOWN;
+	device->policy    = CONNMAN_DEVICE_POLICY_AUTO;
+	device->secondary = FALSE;
 
 	switch (type) {
 	case CONNMAN_DEVICE_TYPE_UNKNOWN:
@@ -996,6 +1057,22 @@ void connman_device_unref(struct connman_device *device)
 	connman_element_unref(&device->element);
 }
 
+const char *__connman_device_get_type(struct connman_device *device)
+{
+	return type2string(device->type);
+}
+
+/**
+ * connman_device_get_type:
+ * @device: device structure
+ *
+ * Get type of device
+ */
+enum connman_device_type connman_device_get_type(struct connman_device *device)
+{
+	return device->type;
+}
+
 /**
  * connman_device_get_name:
  * @device: device structure
@@ -1056,6 +1133,13 @@ void connman_device_set_interface(struct connman_device *device,
 
 	g_free(device->interface);
 	device->interface = g_strdup(interface);
+
+	if (device->name == NULL) {
+		const char *str = type2description(device->type);
+		if (str != NULL && device->interface != NULL)
+			device->name = g_strdup_printf("%s (%s)", str,
+							device->interface);
+	}
 }
 
 /**
@@ -1069,6 +1153,25 @@ const char *connman_device_get_interface(struct connman_device *device)
 	return device->interface;
 }
 
+/**
+ * connman_device_set_ident:
+ * @device: device structure
+ * @ident: unique identifier
+ *
+ * Set unique identifier of device
+ */
+void connman_device_set_ident(struct connman_device *device,
+							const char *ident)
+{
+	g_free(device->ident);
+	device->ident = g_strdup(ident);
+}
+
+const char *__connman_device_get_ident(struct connman_device *device)
+{
+	return device->ident;
+}
+
 /**
  * connman_device_set_policy:
  * @device: device structure
@@ -1106,6 +1209,30 @@ enum connman_device_mode connman_device_get_mode(struct connman_device *device)
 	return device->mode;
 }
 
+/**
+ * connman_device_set_secondary:
+ * @device: device structure
+ * @secondary: secondary value
+ *
+ * Change secondary value of device
+ */
+void connman_device_set_secondary(struct connman_device *device,
+                                                connman_bool_t secondary)
+{
+	device->secondary = secondary;
+}
+
+/**
+ * connman_device_get_secondary:
+ * @device: device structure
+ *
+ * Get secondary value of device
+ */
+connman_bool_t connman_device_get_secondary(struct connman_device *device)
+{
+	return device->secondary;
+}
+
 /**
  * connman_device_set_powered:
  * @device: device structure
@@ -1195,38 +1322,20 @@ int connman_device_set_carrier(struct connman_device *device,
 
 	device->carrier = carrier;
 
-	if (carrier == TRUE) {
-		enum connman_element_type type = CONNMAN_ELEMENT_TYPE_UNKNOWN;
-		struct connman_element *element;
+	return set_carrier(device, device->carrier);
+}
 
-		switch (device->element.ipv4.method) {
-		case CONNMAN_IPV4_METHOD_UNKNOWN:
-		case CONNMAN_IPV4_METHOD_OFF:
-			return 0;
-		case CONNMAN_IPV4_METHOD_STATIC:
-			type = CONNMAN_ELEMENT_TYPE_IPV4;
-			break;
-		case CONNMAN_IPV4_METHOD_DHCP:
-			type = CONNMAN_ELEMENT_TYPE_DHCP;
-			break;
-		}
+int __connman_device_connect(struct connman_device *device)
+{
+	DBG("device %p", device);
 
-		element = connman_element_create(NULL);
-		if (element != NULL) {
-			element->type  = type;
-			element->index = device->element.index;
-
-			if (connman_element_register(element,
-							&device->element) < 0)
-				connman_element_unref(element);
-		}
-	} else
-		connman_element_unregister_children(&device->element);
+	if (device->disconnected == FALSE)
+		return -EINVAL;
 
 	return 0;
 }
 
-void __connman_device_disconnect(struct connman_device *device)
+int __connman_device_disconnect(struct connman_device *device)
 {
 	GHashTableIter iter;
 	gpointer key, value;
@@ -1243,6 +1352,8 @@ void __connman_device_disconnect(struct connman_device *device)
 		if (connman_network_get_connected(network) == TRUE)
 			__connman_network_disconnect(network);
 	}
+
+	return 0;
 }
 
 static void connect_known_network(struct connman_device *device)
@@ -1873,7 +1984,8 @@ update:
 
 	data = g_key_file_to_data(keyfile, &length, NULL);
 
-	g_file_set_contents(pathname, data, length, NULL);
+	if (g_file_set_contents(pathname, data, length, NULL) == FALSE)
+		connman_error("Failed to store device information");
 
 done:
 	g_free(data);
