@@ -33,6 +33,8 @@ struct connman_network {
 	struct connman_element element;
 	enum connman_network_type type;
 	enum connman_network_protocol protocol;
+	connman_bool_t associating;
+	connman_bool_t secondary;
 	connman_bool_t available;
 	connman_bool_t connected;
 	connman_bool_t remember;
@@ -42,6 +44,7 @@ struct connman_network {
 	char *address;
 	char *name;
 	char *node;
+	char *group;
 
 	struct connman_network_driver *driver;
 	void *driver_data;
@@ -118,15 +121,9 @@ static DBusMessage *get_properties(DBusConnection *conn,
 		connman_dbus_dict_append_variant(&dict, "Name",
 					DBUS_TYPE_STRING, &network->name);
 
-	connman_dbus_dict_append_variant(&dict, "Available",
-				DBUS_TYPE_BOOLEAN, &network->available);
-
 	connman_dbus_dict_append_variant(&dict, "Connected",
 				DBUS_TYPE_BOOLEAN, &network->connected);
 
-	connman_dbus_dict_append_variant(&dict, "Remember",
-				DBUS_TYPE_BOOLEAN, &network->remember);
-
 	if (network->priority > 0)
 		connman_dbus_dict_append_variant(&dict, "Priority",
 					DBUS_TYPE_BYTE, &network->priority);
@@ -182,19 +179,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 	type = dbus_message_iter_get_arg_type(&value);
 
-	if (g_str_equal(name, "Remember") == TRUE) {
-		connman_bool_t remember;
-
-		if (type != DBUS_TYPE_BOOLEAN)
-			return __connman_error_invalid_arguments(msg);
-
-		dbus_message_iter_get_basic(&value, &remember);
-
-		if (network->remember == remember)
-			return __connman_error_invalid_arguments(msg);
-
-		network->remember = remember;
-	} else if (g_str_equal(name, "WiFi.Passphrase") == TRUE) {
+	if (g_str_equal(name, "WiFi.Passphrase") == TRUE) {
 		const char *passphrase;
 
 		if (type != DBUS_TYPE_STRING)
@@ -237,7 +222,7 @@ static DBusMessage *do_connect(DBusConnection *conn,
 		return __connman_error_permission_denied(msg);
 
 	if (network->connected == TRUE)
-		return __connman_error_failed(msg);
+		return __connman_error_failed(msg, EALREADY);
 
 	if (network->driver && network->driver->connect) {
 		enum connman_device_mode mode;
@@ -248,7 +233,7 @@ static DBusMessage *do_connect(DBusConnection *conn,
 
 		err = network->driver->connect(network);
 		if (err < 0 && err != -EINPROGRESS)
-			return __connman_error_failed(msg);
+			return __connman_error_failed(msg, -err);
 	} else
 		network->connected = TRUE;
 
@@ -268,7 +253,7 @@ static DBusMessage *do_disconnect(DBusConnection *conn,
 		return __connman_error_permission_denied(msg);
 
 	if (network->connected == FALSE)
-		return __connman_error_failed(msg);
+		return __connman_error_failed(msg, EINVAL);
 
 	connman_element_unregister_children(&network->element);
 
@@ -277,7 +262,7 @@ static DBusMessage *do_disconnect(DBusConnection *conn,
 	if (network->driver && network->driver->disconnect) {
 		err = network->driver->disconnect(network);
 		if (err < 0 && err != -EINPROGRESS)
-			return __connman_error_failed(msg);
+			return __connman_error_failed(msg, -err);
 	} else
 		network->connected = FALSE;
 
@@ -433,6 +418,7 @@ static void network_destruct(struct connman_element *element)
 	g_free(network->wifi.security);
 	g_free(network->wifi.passphrase);
 
+	g_free(network->group);
 	g_free(network->node);
 	g_free(network->name);
 	g_free(network->address);
@@ -454,7 +440,6 @@ struct connman_network *connman_network_create(const char *identifier,
 	connman_uint8_t strength = 0;
 	const char *str;
 	char *temp;
-	unsigned int i;
 
 	DBG("identifier %s type %d", identifier, type);
 
@@ -466,19 +451,13 @@ struct connman_network *connman_network_create(const char *identifier,
 
 	__connman_element_initialize(&network->element);
 
+	//temp = connman_dbus_encode_string(identifier);
 	temp = g_strdup(identifier);
 	if (temp == NULL) {
 		g_free(network);
 		return NULL;
 	}
 
-	for (i = 0; i < strlen(temp); i++) {
-		char tmp = temp[i];
-		if ((tmp < '0' || tmp > '9') && (tmp < 'A' || tmp > 'Z') &&
-						(tmp < 'a' || tmp > 'z'))
-			temp[i] = '_';
-	}
-
 	network->element.name = temp;
 	network->element.type = CONNMAN_ELEMENT_TYPE_NETWORK;
 
@@ -491,7 +470,8 @@ struct connman_network *connman_network_create(const char *identifier,
 
 	connman_element_set_uint8(&network->element, "Strength", strength);
 
-	network->type = type;
+	network->type       = type;
+	network->secondary  = FALSE;
 	network->identifier = g_strdup(identifier);
 
 	return network;
@@ -522,6 +502,22 @@ void connman_network_unref(struct connman_network *network)
 	connman_element_unref(&network->element);
 }
 
+const char *__connman_network_get_type(struct connman_network *network)
+{
+	return type2string(network->type);
+}
+
+/**
+ * connman_network_get_type:
+ * @network: network structure
+ *
+ * Get type of network
+ */
+enum connman_network_type connman_network_get_type(struct connman_network *network)
+{
+	return network->type;
+}
+
 /**
  * connman_network_get_identifier:
  * @network: network structure
@@ -580,6 +576,33 @@ void connman_network_set_protocol(struct connman_network *network,
 	network->protocol = protocol;
 }
 
+/**
+ * connman_network_set_group:
+ * @network: network structure
+ * @group: group name
+ *
+ * Set group name for automatic clustering
+ */
+void connman_network_set_group(struct connman_network *network,
+							const char *group)
+{
+	g_free(network->group);
+	network->group = g_strdup(group);
+}
+
+const char *__connman_network_get_group(struct connman_network *network)
+{
+	return network->group;
+}
+
+const char *__connman_network_get_ident(struct connman_network *network)
+{
+	if (network->device == NULL)
+		return NULL;
+
+	return __connman_device_get_ident(network->device);
+}
+
 /**
  * connman_network_set_available:
  * @network: network structure
@@ -590,10 +613,6 @@ void connman_network_set_protocol(struct connman_network *network,
 int connman_network_set_available(struct connman_network *network,
 						connman_bool_t available)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *key = "Available";
-
 	DBG("network %p available %d", network, available);
 
 	if (network->available == available)
@@ -601,31 +620,6 @@ int connman_network_set_available(struct connman_network *network,
 
 	network->available = available;
 
-	if (network->registered == FALSE)
-		return 0;
-
-	if (network->connected == TRUE)
-		return 0;
-
-	if (network->remember == FALSE)
-		return 0;
-
-	signal = dbus_message_new_signal(network->element.path,
-				CONNMAN_NETWORK_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return 0;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN, &available);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
-
 	return 0;
 }
 
@@ -643,6 +637,9 @@ connman_bool_t connman_network_get_available(struct connman_network *network)
 static gboolean set_connected(gpointer user_data)
 {
 	struct connman_network *network = user_data;
+	struct connman_service *service;
+
+	service = __connman_service_lookup_from_network(network);
 
 	if (network->connected == TRUE) {
 		struct connman_element *element;
@@ -673,8 +670,14 @@ static gboolean set_connected(gpointer user_data)
 			if (connman_element_register(element,
 						&network->element) < 0)
 				connman_element_unref(element);
+
+			__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_CONFIGURATION);
 		}
 	} else {
+		__connman_service_indicate_state(service,
+						CONNMAN_SERVICE_STATE_IDLE);
+
 		connman_element_unregister_children(&network->element);
 
 		__connman_device_set_network(network->device, NULL);
@@ -685,6 +688,34 @@ static gboolean set_connected(gpointer user_data)
 	return FALSE;
 }
 
+/**
+ * connman_network_set_associating:
+ * @network: network structure
+ * @associating: associating state
+ *
+ * Change associating state of network
+ */
+int connman_network_set_associating(struct connman_network *network,
+						connman_bool_t associating)
+{
+	DBG("network %p associating %d", network, associating);
+
+	if (network->associating == associating)
+		return -EALREADY;
+
+	network->associating = associating;
+
+	if (associating == TRUE) {
+		struct connman_service *service;
+
+		service = __connman_service_lookup_from_network(network);
+		__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_ASSOCIATION);
+	}
+
+	return 0;
+}
+
 /**
  * connman_network_set_connected:
  * @network: network structure
@@ -706,6 +737,9 @@ int connman_network_set_connected(struct connman_network *network,
 
 	network->connected = connected;
 
+	if (connected == TRUE)
+		network->associating = FALSE;
+
 	if (network->registered == FALSE) {
 		g_idle_add(set_connected, network);
 		return 0;
@@ -753,10 +787,6 @@ connman_bool_t connman_network_get_connected(struct connman_network *network)
 int connman_network_set_remember(struct connman_network *network,
 						connman_bool_t remember)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *key = "Remember";
-
 	DBG("network %p remember %d", network, remember);
 
 	if (network->remember == remember)
@@ -764,25 +794,6 @@ int connman_network_set_remember(struct connman_network *network,
 
 	network->remember = remember;
 
-	if (network->registered == FALSE)
-		return 0;
-
-	signal = dbus_message_new_signal(network->element.path,
-				CONNMAN_NETWORK_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return 0;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN, &remember);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
-
 	return 0;
 }
 
@@ -1070,7 +1081,21 @@ static int network_probe(struct connman_element *element)
 		return err;
 	}
 
-	__connman_profile_add_network(network);
+	network->secondary = connman_device_get_secondary(network->device);
+
+	switch (network->type) {
+	case CONNMAN_NETWORK_TYPE_UNKNOWN:
+	case CONNMAN_NETWORK_TYPE_VENDOR:
+	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
+	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
+	case CONNMAN_NETWORK_TYPE_HSO:
+		break;
+	case CONNMAN_NETWORK_TYPE_WIFI:
+	case CONNMAN_NETWORK_TYPE_WIMAX:
+		if (network->secondary == FALSE)
+			__connman_profile_add_network(network);
+		break;
+	}
 
 	return 0;
 }
@@ -1087,7 +1112,19 @@ static void network_remove(struct connman_element *element)
 	if (network->driver == NULL)
 		return;
 
-	__connman_profile_remove_network(network);
+	switch (network->type) {
+	case CONNMAN_NETWORK_TYPE_UNKNOWN:
+	case CONNMAN_NETWORK_TYPE_VENDOR:
+	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
+	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
+	case CONNMAN_NETWORK_TYPE_HSO:
+		break;
+	case CONNMAN_NETWORK_TYPE_WIFI:
+	case CONNMAN_NETWORK_TYPE_WIMAX:
+		if (network->secondary == FALSE)
+			__connman_profile_remove_network(network);
+		break;
+	}
 
 	unregister_interface(element);
 
@@ -1174,9 +1211,6 @@ static int network_load(struct connman_network *network)
 
 	g_free(data);
 
-	network->remember = g_key_file_get_boolean(keyfile,
-					network->identifier, "Remember", NULL);
-
 	val = g_key_file_get_integer(keyfile, network->identifier,
 							"Priority", NULL);
 	if (val > 0)
@@ -1232,9 +1266,6 @@ static int network_save(struct connman_network *network)
 	g_free(data);
 
 update:
-	g_key_file_set_boolean(keyfile, network->identifier,
-					"Remember", network->remember);
-
 	if (network->priority > 0)
 		g_key_file_set_integer(keyfile, network->identifier,
 						"Priority", network->priority);
@@ -1251,7 +1282,8 @@ update:
 
 	data = g_key_file_to_data(keyfile, &length, NULL);
 
-	g_file_set_contents(pathname, data, length, NULL);
+	if (g_file_set_contents(pathname, data, length, NULL) == FALSE)
+		connman_error("Failed to store network information");
 
 done:
 	g_free(data);
